import { DominoesGame } from "./DominoesGame";
import { GameSession } from "./GameSession";
import { GameState } from "./dto/GameState";

export class PlayerNotificationManager {
  game!: DominoesGame;
  session!: GameSession;

  constructor(game: DominoesGame | GameSession) {
    if (game instanceof GameSession) {
      this.session = game;
    } else {
      this.game = game; 
    }
  }

  async notifyGameState() {
    if(!this.game) throw new Error('Game not initialized');
    const gameState: GameState = this.game.getGameState();
    const { players } = this.game;
    let promises: Promise<void>[] = players.map(player => player.notifyGameState(gameState));
    return await Promise.all(promises);
  }

  async notifyPlayersState() {
    if(!this.game) throw new Error('Game not initialized');
    const { players } = this.game;
    let promises: Promise<void>[] = players.map(player => player.notifyPlayerState(player.getState()));
    return await Promise.all(promises);
  }


  async notifySessionState() {
    if(!this.session) throw new Error('Session not initialized');
    const { players } = this.session;
    let promises: Promise<void>[] = players.map(player => player.notifySessionState(this.session.getState()));
    return await Promise.all(promises);
  }
}