import { PlayerInteractionInterface } from "../../PlayerInteractionInterface";
import { PlayerInteractionNetwork } from "../../PlayerInteractionNetwork";
import { PlayerHuman } from "./PlayerHuman";
import { NetworkClientNotifier } from "../../NetworkClientNotifier";
import { Tile } from "../Tile";
import { Board } from "../Board";
import { GameState } from "../../dto/GameState";
import { PlayerState } from "../../dto/PlayerState";
import { GameSessionState } from "../../dto/GameSessionState";
import { SocketDisconnectedError } from "../../../common/exceptions/SocketDisconnectedError";

export class NetworkPlayer extends PlayerHuman {
  socketId: string;
  playerInteraction: PlayerInteractionInterface = new PlayerInteractionNetwork(this);
  clientNotifier: NetworkClientNotifier = new NetworkClientNotifier();
  
  constructor(name: string, socketId: string) {
    super(name);
    this.socketId = socketId;
  }

  async notifyGameState(state: GameState): Promise<void> {
    const response = await this.clientNotifier.notifyPlayer(this, 'gameState', state);
    console.log('game state notified :>> ', response);
    if (response === undefined || response.status !== 'ok' ) {
      throw new SocketDisconnectedError();
    }
  } 

  async notifyPlayerState(state: PlayerState): Promise<void> {
    const response = await this.clientNotifier.notifyPlayer(this, 'playerState', state);
    console.log('player state notified :>> ', response);
    if (response === undefined || response.status !== 'ok' ) {
      throw new SocketDisconnectedError();
    }
  }

  async notifySessionState(state: GameSessionState): Promise<void> {
    const response = await this.clientNotifier.notifyPlayer(this, 'sessionState', state);
    console.log('session state notified :>> ', response);
    if (response === undefined || response.status !== 'ok' ) {
      throw new SocketDisconnectedError();
    }
  }
  

  async chooseTile(board: Board): Promise<Tile> {
    return await this.playerInteraction.chooseTile(board);
  }
}