import { LoggingService } from "../../common/LoggingService";
import { GameSession } from "../../game/GameSession";
import { NetworkPlayer } from "../../game/entities/player/NetworkPlayer";

import { ControllerBase } from "./ControllerBase";

export class SessionController extends ControllerBase{
  private static sessions: any = {};

  constructor() {
    super();
    this.logger.info('SessionController created');
  }

  createSession(data: any, socketId: string): any {
    const { user, sessionName } = data;
    const player =  new NetworkPlayer(user, socketId);
    const session = new GameSession(player, sessionName);
    SessionController.sessions[session.id] = session;

    return {
      status: 'ok',
      sessionId: session.id,
      playerId: player.id
    };
  }

  joinSession(data: any, socketId: string): any {
    this.logger.debug('joinSession data :>> ')
    this.logger.object(data);
    const { user, sessionId } = data;
    const session = SessionController.sessions[sessionId];
    const player =  new NetworkPlayer(user, socketId);
    session.addPlayer(player);
    return {
      status: 'ok',
      sessionId: session.id,
      playerId: player.id
    };
  }

  startSession(data: any): any {
    const sessionId: string = data.sessionId;
    const seed: string | undefined = data.seed;
    const session = SessionController.sessions[sessionId];

    if (!session) {
      return ({
        status: 'error',
        message: 'Session not found'
      });
    } else if (session.gameInProgress) {
      return {
        status: 'error',
        message: 'Game already in progress'
      };
    } else {
      const missingHumans = session.maxPlayers - session.numPlayers;
      for (let i = 0; i < missingHumans; i++) {
        session.addPlayer(session.createPlayerAI(i));
      }
      session.start(seed);
      return {
        status: 'ok'
      };
    }
  }


  getSession(id: string) {
    return SessionController.sessions[id];
  }

  deleteSession(id: string) {
    delete SessionController.sessions[id];
  }
}